module Ansjer
{
	sequence<byte> Data;

	struct FrameInfo
	{
		short codecID;
		byte flags;
		byte camIndex;
		byte onlineNum;
		Data reserve1;
		int reserve2;
		long timestamp;
	}

	struct MyFrameInfo
	{
		FrameInfo frameInfo;
		int frameNum;
		long length;
		int frameType;
	}

	struct LanSearchInfo2
	{
		string UID;
		string IP;
		int port;
		string DeviceName;
		byte Reserved; 
	};
	
	struct SMsgAVIoctrlDeviceInfo
	{
        string model;		// IPCam mode
        string vendor;	// IPCam manufacturer
        int version;		// IPCam firmstringware version	ex. v1.2.3.4 => 0x01020304;  v1.0.0.2 => 0x01000002
        int channel;		// Camera index
        int total;			// 0: No cards been detected or an unrecognizeable sdcard that could not be re-formatted.
                                // -1: if camera detect an unrecognizable sdcard, and could be re-formatted
                                // otherwise: return total space size of sdcard (MBytes)

        int free;			// Free space size of sdcard (MBytes)
        string reserved;	// reserved
	};
	struct STimeDay
	{
        short year;	// The number of year.
        short month;	// The number of months since January, in the range 1 to 12.
        short day;		// The day of the month, in the range 1 to 31.
        short wday;		// The number of days since Sunday, in the range 0 to 6. (Sunday = 0, Monday = 1, ...)
        short hour;     // The number of hours past midnight, in the range 0 to 23.
        short minute;   // The number of minutes after the hour, in the range 0 to 59.
        short second;   // The number of seconds after the minute, in the range 0 to 59.
	};

	struct SAvEvent
	{
        STimeDay stTime;
        byte event;
        byte status;				// 0x00: Recording file exists, Event unreaded
                                        // 0x01: Recording file exists, Event readed
                                        // 0x02: No Recording file in the event
        Data reserved;
        int duration;      //file record long
	};


	struct SAvEventOld
	{
        STimeDay stTime;
        byte event;
        byte status;				// 0x00: Recording file exists, Event unreaded
                                        // 0x01: Recording file exists, Event readed
                                        // 0x02: No Recording file in the event
        Data reserved;
	};

	struct playbackRecord
	{
        int channel;	// Camera Index
        int command;	// play record command. refer to ENUM_PLAYCONTROL
        int Param;		// command param, that the user defined
        STimeDay stTimeDay;		// Event time from ListEvent
        Data reserved;
	};

	struct avIoCtrlListEventReq
	{
        int channel; 		// Camera Index
        STimeDay stStartTime; 		// Search event from ...
        STimeDay stEndTime;	  		// ... to (search event)
        byte event;  		// event type, refer to ENUM_EVENTTYPE
        byte status; 		// 0x00: Recording file exists, Event unreaded
                                // 0x01: Recording file exists, Event readed
                                // 0x02: No Recording file in the event
        Data reserved;
	};

	struct getPlaybackReq
	{
		int channel;				  // Camera Index
		STimeDay stTimeDay;   // Event time from ListEvent
		Data reserved;
	};

	sequence<SAvEventOld> sSAvEventOld;
	struct avIoCtrlListEventRespOld
	{
        int  channel;		// Camera Index
        int  total;		// Total event amount in this search session
        byte index;		// package index, 0,1,2...;
								// because avSendIOCtrl() send package up to 1024 bytes one time, you may want split search results to serveral package to send.
        byte endflag;		// end flag; endFlag = 1 means this package is the last one.
		byte count;		// how much events in this package
        Data reserved;
        sSAvEventOld stEvent;		// The first memory address of the events in this package
	};

	sequence<SAvEvent> sSAvEvent;
	struct avIoCtrlListEventResp
	{
        int  channel;		// Camera Index
        int  total;		// Total event amount in this search session
        byte index;		// package index, 0,1,2...;
                                                                // because avSendIOCtrl() send package up to 1024 bytes one time, you may want split search results to serveral package to send.
        byte endflag;		// end flag; endFlag = 1 means this package is the last one.
        byte count;		// how much events in this package
        Data reserved;
        sSAvEvent stEvent;		// The first memory address of the events in this package
	};
	
	// only for play record start command
	struct avIoCtrlPlayRecordResp
	{
        int command;	// Play record command. refer to ENUM_PLAYCONTROL
        int result; 	// Depends on command
							// when is AVIOCTRL_RECORD_PLAY_START:
							//	result>=0   real channel no used by device for playback
							//	result <0	error
							//			-1	playback error
							//			-2	exceed max allow client amount
        Data reserved;
	};

	sequence<LanSearchInfo2> LanDevInfo;

	interface FFMpeg
	{
		bool initDecoder();
		int destroyDecoder(int index);
		int createDecoder(int codecType, int width, int height, int frameRate);
		int decDecode(int index, Data inBuf, int inLen, out Data outBuf, out int width, out int height);
	}
	
	interface AVAPIs
    {        								
		int recvIoData(int avIndex, out int ioCtrlType, out Data buf);
		int sendIoData(int avIndex, int ioCtrlType, Data buf, int len);
		int recvVideoData(int avIndex, out Data buf, out MyFrameInfo outFrameInfo, out int frameNumber);
		int recvAudioData(int avIndex, out Data buf, out MyFrameInfo outFrameInfo, out int frameNumber);

		// The following functions are for testing only
		int init();
		int getAVApiVer();
		int startStream(int avIndex);
		int stopStream(int avIndex);
		void deviceStop(int gSid);
		int connectDevice(string uid);
		void clientStop(int gSid, int avIndex, int channelID);
		int clientStart(int gSid, string uid, int channel, string userName, string userPassword);
	    
		int startLanSearch(int nArrayLen, int nWaitTimeMs, out LanDevInfo lanSearchInfo);
		/*
		int initialize(int nMaxChannelNum);
		int deinitialize();
		int servStart(int nIOTCSessionID, int nTimeout, int nServType, byte nIOTCChannelID, out int nRsend);
		void servStop(int nAVChannelID);
		int servGetResendSize(int avIndex, out int nSize);
		int sendAudioData(int nAVChannelID, Data audioData, int audioDataSize);
		int clientStart(int nIOTCSessionID, string userName, string userPassword, int channelID);
		void clientStop(int nAVChannelID);
		int recvFrameData2(int nAVChannelID, Data frameData, out MyFrameInfo frameInfo);
		int recvAudioData(int nAVChannelID, Data audioData, out MyFrameInfo frameInfo);
		int checkAudioBuf(int nAVChannelID);
		int clientCleanBuf(int nAVChannelID);
		int clientCleanAudioBuf(int nAVChannelID);
		int sendIOCtrl(int nAVChannelID, int nIOCtrlType, Data ioCtrlData, int nIOCtrlDataSize);
		int recvIOCtrl(int nAVChannelID, Data ioCtrlData, int nIOCtrlMaxDataSize, int nTimeeout, out int nIOCtrlType);
		*/
    }
}