const LAN_SEARCH_MAX = 256;
const MAX_RECV_SIZE = 30000;

const CANVAS_MSG = {
  CANVAS_WORKER_INIT_REQ: 0x0001,
  CANVAS_WORKER_INIT_RESP: 0x0002,
  CANVAS_CANVAS_INIT_REQ: 0x0003,
  CANVAS_CANVAS_INIT_RESP: 0x0004,
  CANVAS_RESIZE_REQ: 0x0005,
  CANVAS_RESIZE_RESP: 0x0006,
  CANVAS_UPDATE_REQ: 0x0007,
  CANVAS_UPDATE_RESP: 0x0008,
  CANVAS_SET_PAGE_REQ: 0x0009,
  CANVAS_SET_PAGE_RESP: 0x000A,
  CANVAS_DEV_INIT_REQ: 0x000B,
  CANVAS_DEV_INIT_RESP: 0x000C,
  CANVAS_SPLITMOD_INIT_REQ: 0x000D,
  CANVAS_SPLITMOD_INIT_RESP: 0x000E,
  CANVAS_SELECTWINNO_INIT_REQ: 0x000F,
  CANVAS_SELECTWINNO_INIT_RESP: 0x0010,
  CANVAS_RESETFRAME_REQ: 0x0011,
  CANVAS_RESETFRAME_RESP: 0x0012,
  CANVAS_SET_DISPLOCATION_REQ: 0x0013,
  CANVAS_SET_DISPLOCATION_RESP: 0x0014,
  CANVAS_DECODE_FINISHED: 0x0015,
};

const AVIOCTRL = {
  AVIOCTRL_IPCAM_START_REQ: 0x0001,
  AVIOCTRL_IPCAM_START_RESP: 0x0002,
  AVIOCTRL_IPCAM_STOP_REQ: 0x0003,
  AVIOCTRL_IPCAM_STOP_RESP: 0x0004,
  AVIOCTRL_IPCAM_STOP_STREAM_REQ: 0x0005,
  AVIOCTRL_IPCAM_STOP_STREAM_RESP: 0x0006,
  AVIOCTRL_IPCAM_START_LANSEARCH_REQ: 0x0007,
  AVIOCTRL_IPCAM_START_LANSEARCH_RESP: 0x0008,
  AVIOCTRL_IPCAM_START_LISTEVENT_REQ: 0x0009,
  AVIOCTRL_IPCAM_START_LISTEVENT_RESP: 0x000A,
  AVIOCTRL_IPCAM_START_PLAYBACK_REQ: 0x000B,
  AVIOCTRL_IPCAM_START_PLAYBACK_RESP: 0x000C,
  AVIOCTRL_IPCAM_STOP_PLAYBACK_REQ: 0x000D,
  AVIOCTRL_IPCAM_STOP_PLAYBACK_RESP: 0x000E,
  AVIOCTRL_IPCAM_PLAY_PAUSE_PLAYBACK_REQ: 0x000F,
  AVIOCTRL_IPCAM_PLAY_PAUSE_PLAYBACK_RESP: 0x0010,
  AVIOCTRL_IPCAM_PLAY_FORWARD_PLAYBACK_REQ: 0x0011,
  AVIOCTRL_IPCAM_PLAY_FORWARD_PLAYBACK_RESP: 0x0012,
  AVIOCTRL_IPCAM_PLAY_BACKWARD_PLAYBACK_REQ: 0x0013,
  AVIOCTRL_IPCAM_PLAY_BACKWARD_PLAYBACK_RESP: 0x0014,
  AVIOCTRL_IPCAM_CHANGE_CHANNEL_QUALITY_REQ: 0x0015,
  AVIOCTRL_IPCAM_UPDATE_CHANNEL_QUALITY_RESP: 0x0016,
  AVIOCTRL_IPCAM_GET_DEVICE_VERSION_RESP: 0x0017,
  AVIOCTRL_IPCAM_SOFTWAREVERSION_RESP: 0x0018,
  AVIOCTRL_IPCAM_GET_PLAYBACK_STATUS_RESP: 0x0019,

  AVIOCTRL_START_INIT_WORKER_REQ: 0x00FFF,
  AVIOCTRL_START_INIT_WORKER_RESP: 0x01000,
  AVIOCTRL_START_INIT_DATA_REQ: 0x01001,
  AVIOCTRL_START_INIT_DATA_RESP: 0x01002,
  AVIOCTRL_STOP_WORKER_REQ: 0x01003,
  AVIOCTRL_STOP_WORKER_RESP: 0x01004,
  AVIOCTRL_SET_DEVICE_ONLINE_RESP: 0x01005,
  AVIOCTRL_SET_CHANNEL_ONLINE_RESP: 0x01006,
  AVIOCTRL_IPCAM_UPDATE_CHANNEL_RESOLUTION_RESP: 0x01007,

  AVIOCTRL_CONNECT_FAIL_RESP: 0x01008,
  AVIOCTRL_MOD_CHANGE_REQ: 0x01009,
  AVIOCTRL_MOD_CHANGE_RESP: 0x0100A,
};

const APP_MSG = {
  START_INIT_WORKER_REQ: 0x10,
  START_INIT_WORKER_RESP: 0x11,
  START_CONN_REQ: 0x01F,
  START_CONN_RESP: 0x020,
  STOP_CONN_REQ: 0x021,
  STOP_CONN_RESP: 0x022,
  START_DEVCLIENT_REQ: 0x023,
  START_DEVCLIENT_RESP: 0x024,
  STOP_DEVCLIENT_REQ: 0x025,
  STOP_DEVCLIENT_RESP: 0x026,
  START_AVCOMM_REQ: 0x027,
  START_AVCOMM_RESP: 0x028,
  STOP_AVCOMM_REQ: 0x029,
  STOP_AVCOMM_RESP: 0x02A,
  SEND_IO_DATA_REQ: 0x02B,
  SEND_IO_DATA_RESP: 0x02C,
  RECV_IO_DATA_REQ: 0x02D,
  RECV_IO_DATA_RESP: 0x02E,
  STOP_WORKER_REQ: 0x02F,
  STOP_WORKER_RESP: 0x030,
  RECV_AV_DATA_REQ: 0x031,
  RECV_AV_DATA_RESP: 0x032,
  START_CREATE_DEC_REQ: 0x033,
  START_CREATE_DEC_RESP: 0x034,
  START_DESTROY_DEC_REQ: 0x035,
  START_DESTROY_DEC_RESP: 0x036,
  START_LANSEARCH_REQ: 0x037,
  START_LANSEARCH_RESP: 0x038,
  START_CONN_REQ2: 0x039,
  START_CONN_RESP2: 0x3A,
  START_DEVCLIENT_REQ2: 0x3B,
  START_DEVCLIENT_RESP2: 0x3C,
  IPCAM_GET_FRAMERATE: 0x3D,
  IPCAM_STOP_DEVICE_REQ: 0x3E,
  IPCAM_STOP_DEVICE_RESP: 0x03F,
  IPCAM_RECV_AVINDEX_REQ: 0x040,
  IPCAM_RECV_AVINDEX_RESP: 0x041,
  IPCAM_LISTEVENT_REQ: 0x0318,
  IPCAM_LISTEVENT_RESP: 0x0319,
  RECORD_PLAYCONTROL_REQ: 0x031A,
  RECORD_PLAYCONTROL_RESP: 0x031B,
  IPCAM_DEVINFO_REQ: 0x0330,
  IPCAM_DEVINFO_RESP: 0x0331,
  IPCAM_GET_PLAYBACK_REQ: 0x033A,
  IPCAM_GET_PLAYBACK_RESP: 0x033B,
  START_INIT_DATA_REQ: 0x0FFF,
  START_INIT_DATA_RESP: 0x1000,
  START_DEC_VIDEO_REQ: 0x1001,
  START_DEC_VIDEO_RESP: 0x1002,
  SHOW_ERRORMSG_REQ: 0x1003,
  SHOW_ERRORMSG_RESP: 0x1004,
  GET_VIDEO_DATA_REQ: 0x1005,
  GET_VIDEO_DATA_RESP: 0x1006,
};

const dev_status = {
  CONN_STATE_NONE: 0,
  CONN_STATE_DISCONNECTED: 1,
  CONN_STATE_CONNECTED: 2,
  CONN_STATE_CHSTOP: 3,
  CONN_STATE_CHSTART: 4,
  CONN_STATE_AVLIVE: 5,
  CONN_STATE_ERROR: 6,
};

/* FRAME Flag */
const frame_flag = {
  IPC_FRAME_FLAG_PBFRAME: 0x00,	        // A/V P/B frame..I
  IPC_FRAME_FLAG_IFRAME: 0x01,	            // A/V I frame.
  IPC_FRAME_FLAG_MD: 0x02,	                 // For motion detection.
  IPC_FRAME_FLAG_IO	: 0x03                    // For Alarm IO detection.
};

/* CODEC ID */
const codecID = {
  MEDIA_CODEC_UNKNOWN: 0x00,
  MEDIA_CODEC_VIDEO_MPEG4: 0x4C,
  MEDIA_CODEC_VIDEO_H263: 0x4D,
  MEDIA_CODEC_VIDEO_H264: 0x4E,
  MEDIA_CODEC_VIDEO_MJPEG: 0x4F,
  MEDIA_CODEC_VIDEO_HEVC: 0x50,
  MEDIA_CODEC_AUDIO_AAC_RAW: 0x86,   // 2017-05-04 add AAC Raw data audio codec definition
  MEDIA_CODEC_AUDIO_AAC_ADTS: 0x87,   // 2017-05-04 add AAC ADTS audio codec definition
  MEDIA_CODEC_AUDIO_AAC_LATM: 0x88,   // 2017-05-04 add AAC LATM audio codec definition
  MEDIA_CODEC_AUDIO_AAC: 0x88,   // 2014-07-02 add AAC LATM audio codec definition
  MEDIA_CODEC_AUDIO_G711U: 0x89,   //g711 u-law
  MEDIA_CODEC_AUDIO_G711A: 0x8A,   //g711 a-law
  MEDIA_CODEC_AUDIO_ADPCM: 0X8B,
  MEDIA_CODEC_AUDIO_PCM: 0x8C,
  MEDIA_CODEC_AUDIO_SPEEX: 0x8D,
  MEDIA_CODEC_AUDIO_MP3: 0x8E,
  MEDIA_CODEC_AUDIO_G726: 0x8F,
};

/* Play Record Command */
const PLAY_CONTROL = {
  RECORD_PLAY_PAUSE: 0x00,
  RECORD_PLAY_STOP: 0x01,
  RECORD_PLAY_STEPFORWARD: 0x02, //now, APP no use
  RECORD_PLAY_STEPBACKWARD: 0x03, //now, APP no use
  RECORD_PLAY_FORWARD: 0x04, //now, APP no use
  RECORD_PLAY_BACKWARD: 0x05, //now, APP no use
  RECORD_PLAY_SEEKTIME: 0x06, //now, APP no use
  RECORD_PLAY_END: 0x07,
  RECORD_PLAY_START: 0x10,
};

Date.prototype.dateFormat = function(fmt) {
  const o = {
    'M+' : this.getMonth() + 1,                 // 月份
    'd+' : this.getDate(),                    // 日
    'h+' : this.getHours(),                   // 小时
    'm+' : this.getMinutes(),                 // 分
    's+' : this.getSeconds(),                 // 秒
    'q+' : Math.floor((this.getMonth() + 3) / 3), // 季度
    'S'  : this.getMilliseconds()             // 毫秒
  };
  if(/(y+)/.test(fmt))
    fmt=fmt.replace(RegExp.$1, (this.getFullYear() + '').substr(4 - RegExp.$1.length));
  for(let k in o)
    if(new RegExp('(' +  k  + ')').test(fmt))
      fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
  return fmt;
};
// console.log('', new Date().dateFormat('yyyy-MM-dd hh:mm:ss.S'));

function typeOf(obj) {
  return toString.call(obj).slice(8, -1);
}

/**
 * @return {string}
 */
function Uint8ArrayToString(uint8Array) {
  let value = '';
  for (let index = 0; index < uint8Array.length; index++) {
    if (uint8Array[index] !== 0)
      value += String.fromCharCode(uint8Array[index]);
  }

  return value;
}

/**
 * @return {boolean}
 */
function Uint8ArrayToBoolean(uint8Array) {
  let value = false;
  if (uint8Array === 1 || uint8Array[0] === 1)
    value = true;
  else if (uint8Array === 0 || uint8Array[0] === 0)
    value = false;
  return value;
}

/**
 * @return {number}
 */
function Uint8ArrayToNumber(uint8Array, len = 4) {
  let value = 0;
  if (len === 1) {
    value = uint8Array[0] & 0xFF;
  }
  else if (len === 2) {
    value = (uint8Array[0] & 0xFF) | ((uint8Array[1] & 0xFF) << 8);
  }
  else if (len === 3) {
    value = (uint8Array[0] & 0xFF) | ((uint8Array[1] & 0xFF) << 8)
      | ((uint8Array[2] & 0xFF) << 16);
  }
  else if (len === 4) {
    value = (uint8Array[0] & 0xFF) | ((uint8Array[1] & 0xFF) << 8)
      | ((uint8Array[2] & 0xFF) << 16) | ((uint8Array[3] & 0xFF) << 24);
  }
  return value;
}

/**
 * @return {number}
 */
function Uint8ArrayToClass(uint8Array, classObj, lenArray, posIndex = 0) {
  let typeArray = Object.keys(classObj);
  typeArray.forEach((value, index) => {
    let type = typeOf(Object.values(classObj)[index]);
    if (type === 'Boolean') {
      classObj[value] = Uint8ArrayToBoolean(uint8Array.slice(posIndex, posIndex + 1));
      posIndex += 1;
    }
    else if (type === 'Number') {
      classObj[value] = Uint8ArrayToNumber(uint8Array.slice(posIndex, posIndex + lenArray[index]), lenArray[index]);
      posIndex += lenArray[index];
    }
    else if (type === 'String') {
      classObj[value] = Uint8ArrayToString(uint8Array.slice(posIndex, posIndex + lenArray[index]));
      posIndex += lenArray[index];
    }
    else if (type === 'Array' || type === 'Uint8Array') {
      type = typeOf(classObj[value][0]);
      if (type === 'Number') {
        classObj[value] = uint8Array.slice(posIndex, posIndex + lenArray[index]);
        posIndex += lenArray[index];
      }
      else if (type === 'String') {
        classObj[value].forEach((value, index2) => {
          const newArray = lenArray[index];
          classObj[value] = Uint8ArrayToNumber(uint8Array.slice(posIndex, posIndex + newArray[index2]), newArray[index2]);
          posIndex += newArray[index2];
        });
      }
      else if (type === 'Object') {
        classObj[value].forEach(value => {
          posIndex = Uint8ArrayToClass(uint8Array, value, lenArray[index], posIndex);
        });
      }
    }
    else if (type === 'Object') {
      if (classObj[value].constructor.name === 'Long') {
        const high =  Uint8ArrayToNumber(uint8Array.slice(posIndex, posIndex + 4), 4);
        const low = Uint8ArrayToNumber(uint8Array.slice(posIndex + 4, posIndex + 8), 4);
        classObj[value].high = high >= 0 ? high: 0;
        classObj[value].low = low >= 0 ? low: 0;
        posIndex += lenArray[index];
      }
      else {
        posIndex = Uint8ArrayToClass(uint8Array, classObj[value], lenArray[index], posIndex);
      }
    }
    else if (type === 'Null') {}
    else if (type === 'Undefined') {}
    else {}
  });

  return posIndex;
}

function stringToUint8Array(str, len){
  let arr = [];
  const uint8Array = new Uint8Array(len);
  for (let index = 0; index  < str.length; ++index) {
    arr.push(str.charCodeAt(index));
  }
  uint8Array.set(arr);

  return uint8Array;
}

function numberToBytes(value, len){
  const byte = new Uint8Array(len);
  byte[0] = value  & 0xFF;
  if (len === 1) {
    byte[1] = (value >> 8) & 0xFF;
  }
  else if (len === 2) {
    byte[1] = (value >> 8) & 0xFF;
    byte[2] = (value >> 16) & 0xFF;
  }
  else if (len === 3) {
    byte[1] = (value >> 8) & 0xFF;
    byte[2] = (value >> 16) & 0xFF;
    byte[3] = (value >> 24) & 0xFF;
  }

  return byte;
}

function booleanToByte(value){
  return value === true ? 1 : 0;
}

/**
 * @return {number}
 */
function ClassToUint8Array(classObj, uint8Array, lenArray, pos = 0) {
  let typeArray = Object.keys(classObj);
  typeArray.forEach((value, index) => {
    let type = typeOf(Object.values(classObj)[index]);
    if (type === 'Boolean') {
      uint8Array.set([booleanToByte(classObj[value])], pos);
      pos += 1;
    }
    else if (type === 'Number') {
      uint8Array.set(numberToBytes(classObj[value], lenArray[index]), pos);
      pos += lenArray[index];
    }
    else if (type === 'String') {
      uint8Array.set(stringToUint8Array(classObj[value], lenArray[index]), pos);
      pos += lenArray[index];
    }
    else if (type === 'Array' || type === 'Uint8Array') {
      type = typeOf(classObj[value][0]);
      if (type === 'Number') {
        uint8Array.set(classObj[value], pos);
        pos += lenArray[index];
      }
      else if (type === 'String') {
        const array = lenArray[index];
        classObj[value].forEach( (value, index) => {
          uint8Array.set(stringToUint8Array( value, array[index]), pos);
          pos += array[index];
        });
      }
      else if (type === 'Object') {
        classObj[value].forEach(value => {
          pos = ClassToUint8Array(value, uint8Array, lenArray[index], pos);
        });
      }
      else {}
    }
    else if (type === 'Object') {
      pos = ClassToUint8Array(classObj[value], uint8Array, lenArray[index], pos);
    }
    else if (type === 'Null') {}
    else if (type === 'Undefined') {}
    else {}
  });

  return pos;
}
