importScripts( 'MsgType.js');
const SPLIT_MODE = {
  SPLIT_MODE_ONEXONE:1,
  SPLIT_MODE_TWOXTWO:4,
  SPLIT_MODE_ONEAFIVE:6,
  SPLIT_MODE_ONEASEVEN:8,
  SPLIT_MODE_THREEXTHREE:9,
  SPLIT_MODE_ONEATWELVE:13,
  SPLIT_MODE_FOURXFOUR:16,
  SPLIT_MODE_FIVEXFIVE:25,
  SPLIT_MODE_SIXXSIX:36,
  SPLIT_MODE_SEVENXSEVEN:49,
  SPLIT_MODE_EIGHTXEIGHT:64,
}

class Scene{
  constructor(){
    this.canvas = undefined;
    this.textureY = undefined;
    this.textureU = undefined;
    this.textureV = undefined;

    this.pboBuffer = undefined;
    this.vertexBuffer = undefined;
    this.textureBuffer = undefined;

    this.vertexIn = undefined;
    this.textureIn = undefined;
    this.pShaderProgram = undefined;

    // @ts-ignore
    this.gl = undefined;

    this._nCurSelectWinNo = undefined;
    this._dispLocation = undefined;
    this._nCurSplitMode = undefined;

    this.busy = false;
    this.frames = 0;
    this.page = 0;
    this.nCurSplitMode = SPLIT_MODE.SPLIT_MODE_TWOXTWO;
    this._nCurSelectWinNo = 0;
    this.pboArray = new Object();
    postMessage([CANVAS_MSG.CANVAS_WORKER_INIT_RESP]);
    return;
  }

  getLocation(location, nMode, nWinNO) {
    let loc;
    let valueOne;
    if (location.locationMaps.size > 0) {
      location.locationMaps.forEach((value, index) => {
        if (index === nMode) {
          if (nMode === SPLIT_MODE.SPLIT_MODE_ONEXONE) {
            loc = value[0];
            return;
          }
          for (let n = 0, len = value.length; n < len; n++) {
            valueOne = value[n];
            if (valueOne.nWinNO === nWinNO) {
              loc = valueOne;
              return;
            }
          }
        }
      });
      return loc;
    }
  }

  createCanvas(canvasOffscreen) {
    this.canvas = canvasOffscreen;
    this.gl = this.canvas.getContext('webgl2',{preserveDrawingBuffer: true});
    this.pboBuffer = this.gl.createBuffer();
    this.gl.bindBuffer(this.gl.PIXEL_UNPACK_BUFFER , this.pboBuffer);
    this.initializeGL();
    this.drawContours();
    postMessage([CANVAS_MSG.CANVAS_CANVAS_INIT_RESP]);
    return;
  }

  initializeGL() {
    // Initialize a shader program;
    this.pShaderProgram = this.initShaderProgram();
    // 绑定属性vertexIn到指定位置ATTRIB_VERTEX,该属性在顶点着色源码其中有声明
    this.gl.bindAttribLocation(this.pShaderProgram, 0, 'vertexIn');
    // 绑定属性textureIn到指定位置ATTRIB_TEXTURE,该属性在顶点着色源码其中有声明
    this.gl.bindAttribLocation(this.pShaderProgram, 1, 'textureIn');

    // 读取着色器中的数据变量tex_y, tex_u, tex_v的位置,这些变量的声明可以在
    // 片段着色器源码中可以看到
    const textY = this.gl.getUniformLocation(this.pShaderProgram, 'tex_y');
    const textU = this.gl.getUniformLocation(this.pShaderProgram, 'tex_u');
    const textV = this.gl.getUniformLocation(this.pShaderProgram, 'tex_v');

    // 获取给定WebGLProgram中的属性变量的位置
    this.vertexIn = this.gl.getAttribLocation(this.pShaderProgram, 'vertexIn');
    this.textureIn = this.gl.getAttribLocation(this.pShaderProgram, 'textureIn');

    // Here's where we call the routine that builds all the objects we'll be drawing.
    this.initBuffers();

    // 创建Y、U、V纹理
    this.textureY = this.gl.createTexture();
    this.textureU = this.gl.createTexture();
    this.textureV = this.gl.createTexture();

    // 将0号纹理传递给着色器
    // 0对应纹理单元GL_TEXTURE0 1对应纹理单元GL_TEXTURE1 2对应纹理的单元
    this.gl.uniform1i(textY, 0);
    this.gl.uniform1i(textU, 1);
    this.gl.uniform1i(textV, 2);

    // 设置背景色
    this.gl.clearColor(0.0, 0.0, 0.0, 0.0);
    this.gl.viewport(0, 0, 640, 480);
    this.gl.clear(this.gl.COLOR_BUFFER_BIT);


    return true;
  }

  initBuffers() {
    // 顶点矩阵
    const vertexVertices = [
      -1.0, -1.0,
      1.0, -1.0,
      -1.0, 1.0,
      1.0, 1.0,
    ];

    // 纹理矩阵
    const textureVertices = [
      0.0,  1.0,
      1.0,  1.0,
      0.0,  0.0,
      1.0,  0.0,
    ];

    this.vertexBuffer = this.gl.createBuffer();
    this.gl.bindBuffer(this.gl.ARRAY_BUFFER, this.vertexBuffer);
    this.gl.bufferData(this.gl.ARRAY_BUFFER, new Float32Array(vertexVertices),
      this.gl.STATIC_DRAW);
    this.gl.vertexAttribPointer(this.vertexIn, 2, this.gl.FLOAT, false,
      0, vertexVertices);

    this.textureBuffer = this.gl.createBuffer();
    this.gl.bindBuffer(this.gl.ARRAY_BUFFER, this.textureBuffer);
    this.gl.bufferData(this.gl.ARRAY_BUFFER, new Float32Array(textureVertices),
      this.gl.STATIC_DRAW);
    this.gl.vertexAttribPointer(this.textureIn, 2, this.gl.FLOAT, false,
      0, textureVertices);

    this.gl.enableVertexAttribArray(this.vertexIn);
    this.gl.enableVertexAttribArray(this.textureIn);

    return true;
  }

  createShader(sourceCode, type) {
    /* 编译着色器 三步*/
    // (1)创建Shader对象
    const shader = this.gl.createShader(type);
    // (2)将着色器附加到Shader上
    this.gl.shaderSource(shader, sourceCode);
    // (3)编译程序
    this.gl.compileShader(shader);

    if (!this.gl.getShaderParameter(shader, this.gl.COMPILE_STATUS)) {
      // const info = this.webGL.getShaderInfoLog( shader );
      // console.log('Could not compile WebGL program: ', info);
      return null;
    }

    return shader;
  }

  initShaderProgram() {
    let pVSHader;
    let pFSHader;
    let pTriangleVSHader;
    let pTriangleFSHader;
    let pShaderProgram;

    let vsSource;  // 顶点着色器源码
    let fsSource;  // 片段着色器源码
    let vertexShaderSource;  // 着色器语言
    let fragmentShaderSource;  // 着色器语言

    vsSource = `
    attribute vec4 vertexIn; \
    attribute vec2 textureIn; \
    varying vec2 textureOut;  \
    void main(void)           \
    {                         \
        gl_Position = vertexIn; \
        textureOut = textureIn; \
    }`;

    fsSource = `
    varying vec2 textureOut; \
    uniform sampler2D tex_y; \
    uniform sampler2D tex_u; \
    uniform sampler2D tex_v; \
    void main(void) \
    { \
        vec3 yuv; \
        vec3 rgb; \
        yuv.x = texture2D(tex_y, textureOut).r; \
        yuv.y = texture2D(tex_u, textureOut).r - 0.5; \
        yuv.z = texture2D(tex_v, textureOut).r - 0.5; \
        rgb = mat3( 1, 1, 1,
                    0, -0.39465,  2.03211,
                    1.13983, -0.58060, 0) * yuv;
        gl_FragColor = vec4(rgb, 1);
    }`;

    vertexShaderSource = `
    attribute highp vec4 posAttr;
    attribute lowp vec4 colAttr;
    varying lowp vec4 col;
    uniform highp mat4 matrix;
    void main() {
       col = colAttr;
       gl_Position = matrix * posAttr;
    }`;

    fragmentShaderSource = `
    varying lowp vec4 col;
    void main() {
       gl_FragColor = col;
    }`;

    pTriangleVSHader = this.createShader(vertexShaderSource,
      this.gl.VERTEX_SHADER);
    if (pTriangleVSHader == null) {
      vertexShaderSource = `
      precision mediump float;
      attribute highp vec4 posAttr;
      attribute lowp vec4 colAttr;
      varying lowp vec4 col;
      uniform highp mat4 matrix;
      void main() {
         col = colAttr;
         gl_Position = matrix * posAttr;
       }`;
      pTriangleVSHader = this.createShader(vertexShaderSource,
        this.gl.VERTEX_SHADER);
    }

    pTriangleFSHader = this.createShader(fragmentShaderSource,
      this.gl.FRAGMENT_SHADER);
    if (pTriangleFSHader == null) {
      fragmentShaderSource = `
      precision mediump float;
      varying lowp vec4 col;
      void main() {
         gl_FragColor = col;
      }`;
      pTriangleFSHader = this.createShader(fragmentShaderSource,
        this.gl.FRAGMENT_SHADER);
    }

    pVSHader = this.createShader(vsSource, this.gl.VERTEX_SHADER);
    if (pVSHader == null) {
      vsSource = `
      precision mediump float;
      attribute vec4 vertexIn;
      attribute vec2 textureIn;
      varying vec2 textureOut;
      void main(void)
      {
          gl_Position = vertexIn; \\
          textureOut = textureIn; \\
       }`;
    }

    pFSHader = this.createShader(fsSource, this.gl.FRAGMENT_SHADER);
    if (pFSHader == null) {
      fsSource = `
      precision mediump float; \
      varying vec2 textureOut; \
      uniform sampler2D tex_y; \
      uniform sampler2D tex_u; \
      uniform sampler2D tex_v; \
      void main(void) \
      { \
          vec3 yuv; \
          vec3 rgb; \
          yuv.x = texture2D(tex_y, textureOut).r; \
          yuv.y = texture2D(tex_u, textureOut).r - 0.5; \
          yuv.z = texture2D(tex_v, textureOut).r - 0.5; \
          rgb = mat3( 1,       1,         1, \
                      0,       -0.39465,  2.03211, \
                      1.13983, -0.58060,  0) * yuv; \
          gl_FragColor = vec4(rgb, 1); \
      }`;

      pFSHader = this.createShader(fsSource, this.gl.FRAGMENT_SHADER);
    }
    // 创建着色器程序容器
    pShaderProgram = this.gl.createProgram();
    // 将片段着色器添加到程序容器
    this.gl.attachShader(pShaderProgram, pVSHader);
    // 将顶点着色器添加到程序容器
    this.gl.attachShader(pShaderProgram, pFSHader);
    // 链接着色器(顶点和片段着色器)
    this.gl.linkProgram(pShaderProgram);
    // 使用程序
    this.gl.useProgram(pShaderProgram);

    if (!this.gl.getProgramParameter(pShaderProgram, this.gl.LINK_STATUS)) {
      // const error = this.webGL.getProgramInfoLog(shaderProgram);
      // console.log('Unable to initialize the shader program: ', error);
      return null;
    }

    return pShaderProgram;
  }

  drawContours() {

    // 刷白整个屏幕
    this.gl.clearColor(1.0, 1.0, 1.0, 1.0);
    this.gl.clear(this.gl.COLOR_BUFFER_BIT);
    for (let i = 0; i < this.nCurSplitMode; i++) {
      const loc = this.getLocation(this.dispLocation, this.nCurSplitMode, i);

      // 高亮当前选中窗口

      if (i === this._nCurSelectWinNo) {
        this.gl.enable(this.gl.SCISSOR_TEST);
        this.gl.scissor(loc.nStartPos_x - 2, loc.nStartPos_y - 2,
          loc.nWidth + 4, loc.nHeight + 4);
        this.gl.clearColor(0.0, 0.5, 1.0, 0.7);
        this.gl.clear(this.gl.COLOR_BUFFER_BIT);
        this.gl.disable(this.gl.SCISSOR_TEST);
      }

      // 刷黑每一个子窗口
      this.gl.enable(this.gl.SCISSOR_TEST);
      this.gl.scissor(loc.nStartPos_x, loc.nStartPos_y, loc.nWidth, loc.nHeight );
      this.gl.clearColor(0.0, 0.0, 0.0, 0.0);
      this.gl.clear(this.gl.COLOR_BUFFER_BIT);
      this.gl.disable(this.gl.SCISSOR_TEST);
    }
    // this.pboArray=[];
    for(const win in this.pboArray){
      this.drawVideo(this.pboArray[win]['channel'], this.pboArray[win]['pboData'], this.pboArray[win]['uid'], this.pboArray[win]['channelid'], this.pboArray[win]['needload'], this.pboArray[win]['isCreated']);
    }
  }

  drawVideo(channel, pboData, uid, channelid, needload, isCreated) {
    const nVideoW = channel._videoStreamInfoLive._width;
    const nVideoH = channel._videoStreamInfoLive._height;
    if (channel._connStatusLive  && nVideoW !== 0 && nVideoH !== 0) {
      const winNo = channel._dispWinNumLive - this.page * this.nCurSplitMode;
      this.pboArray[winNo] = {channel: channel, pboData: pboData, uid: uid, channelid: channelid, needload: needload, isCreated: isCreated};
      if (this._nCurSplitMode === SPLIT_MODE.SPLIT_MODE_ONEXONE && this._nCurSelectWinNo !== channel._dispWinNumLive) {
        return;
      }
      if (channel._dispWinNumLive - this.page * this.nCurSplitMode >= this.nCurSplitMode) {
        return;
      }
      const loc = this.getLocation(this.dispLocation, this._nCurSplitMode, winNo);
      if (!loc) {
        return;
      }
      if (!isCreated) {
        this.gl.bufferData(this.gl.PIXEL_UNPACK_BUFFER, new ArrayBuffer(1.5 * 3840 * 2160), this.gl.DYNAMIC_DRAW);
      }
      this.gl.bufferSubData(this.gl.PIXEL_UNPACK_BUFFER, 0, pboData);
      this.gl.viewport(loc.nStartPos_x, loc.nStartPos_y, loc.nWidth, loc.nHeight);
      this.loadY(nVideoW, nVideoH);
      this.loadU(nVideoW, nVideoH);
      this.loadV(nVideoW, nVideoH);
      // this.gl.generateMipmap(this.gl.TEXTURE_2D );
      // 使用顶点数组方式绘制图形
      this.gl.drawArrays(this.gl.TRIANGLE_STRIP, 0, 4);
      this.frames++;
      postMessage([CANVAS_MSG.CANVAS_UPDATE_RESP, [this.frames, uid, channelid, needload, isCreated]]);
    }
    return;
  }

  loadY(nVideoW, nVideoH) {
    // 加载y数据纹理
    this.gl.activeTexture(this.gl.TEXTURE0); // 激活纹理单元GL_TEXTURE0
    this.gl.bindTexture(this.gl.TEXTURE_2D, this.textureY); // 使用来自y数据生成纹理
    this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.LUMINANCE,
      nVideoW, nVideoH, 0, this.gl.LUMINANCE, this.gl.UNSIGNED_BYTE, 0);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MAG_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MIN_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S,
      this.gl.CLAMP_TO_EDGE);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T,
      this.gl.CLAMP_TO_EDGE);
    return;
  }

  loadU(nVideoW, nVideoH) {
    // 加载u数据纹理
    this.gl.activeTexture(this.gl.TEXTURE1);
    this.gl.bindTexture(this.gl.TEXTURE_2D, this.textureU);
    this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.LUMINANCE,
      nVideoW / 2, nVideoH / 2, 0, this.gl.LUMINANCE,
      this.gl.UNSIGNED_BYTE, nVideoW * nVideoH);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MAG_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MIN_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S,
      this.gl.CLAMP_TO_EDGE);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T,
      this.gl.CLAMP_TO_EDGE);
    return;
  }

  loadV(nVideoW, nVideoH) {
    // 加载v数据纹理
    this.gl.activeTexture(this.gl.TEXTURE2);
    this.gl.bindTexture(this.gl.TEXTURE_2D, this.textureV);
    this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.LUMINANCE,
      nVideoW / 2, nVideoH / 2, 0, this.gl.LUMINANCE,
      this.gl.UNSIGNED_BYTE, nVideoW * nVideoH * 5 / 4);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MAG_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MIN_FILTER,
      this.gl.LINEAR);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S,
      this.gl.CLAMP_TO_EDGE);
    this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T,
      this.gl.CLAMP_TO_EDGE);
    return;
  }

  updateRequest(type = 'cleanDev', channelItem = null, pboData = null, uid = null, channelid = null, needload = false, isCreated = false) {
    if (type === 'cleanDev') {
      this.drawContours();
      return;
    } else {
      this.drawVideo(channelItem, pboData, uid , channelid, needload, isCreated);
      return;
    }
  }

  get nCurSelectWinNo() {
    return this._nCurSelectWinNo;
  }

  set nCurSelectWinNo(value) {
    this._nCurSelectWinNo = value;
  }

  get nCurSplitMode() {
    return this._nCurSplitMode;
  }

  set nCurSplitMode(value) {
    this._nCurSplitMode = value;
  }

  get dispLocation() {
    return this._dispLocation;
  }

  set dispLocation(value) {
    this._dispLocation = value;
  }
}

let scene = new Scene();

self.addEventListener('message', msg => {
  const control = msg.data[0];
  const resultValue = msg.data[1];
  switch (control) {
    case CANVAS_MSG.CANVAS_CANVAS_INIT_REQ:
      scene.dispLocation = resultValue[1];
      scene.page = resultValue[2];
      scene.createCanvas(resultValue[0]);
      break;

    case CANVAS_MSG.CANVAS_SET_DISPLOCATION_REQ:
      scene.dispLocation = resultValue[0];
      scene.canvas.width = resultValue[1];
      scene.canvas.height = resultValue[2];
      scene.updateRequest();
      break;

    case CANVAS_MSG.CANVAS_UPDATE_REQ:
      if(this.busy){
        return;
      }
      this.busy = true;
      scene.updateRequest(resultValue[0], resultValue[1], resultValue[2], resultValue[3], resultValue[4], resultValue[5], resultValue[6]);
      if(resultValue[0]==="cleanDev"){
        this.pboArray = new Object();
      }
      this.busy = false;
      break;

    case CANVAS_MSG.CANVAS_SET_PAGE_REQ:
      scene.page = resultValue;
      if(scene.canvas !== undefined)
        scene.updateRequest();
      break;

    case CANVAS_MSG.CANVAS_SPLITMOD_INIT_REQ:
      scene.nCurSplitMode = resultValue;
      if(scene.canvas !== undefined)
        scene.updateRequest();
      break;

    case CANVAS_MSG.CANVAS_SELECTWINNO_INIT_REQ:
      scene.nCurSelectWinNo = resultValue;
      if(scene.canvas !== undefined)
        scene.updateRequest();
      break;

    case CANVAS_MSG.CANVAS_RESETFRAME_REQ:
      scene.frames = 0;
      break;
  }
});
